/*
 *      Program do przegladania logow z ekg.
 *      Klasa bazowa dla obiektu komendy.
 *
 *
 *      copyright (c) 2003 by Lukasz Bartnik
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License Version 2 as
 *  published by the Free Software Foundation.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#ifndef __COMMAND_HH__
#define __COMMAND_HH__

#include <string>
#include <vector>

/*
 * klasa pomocnicza - niektore komendy rzucaja wyjatki
 * pochodne od tej klasy
 */
class command_exception {
public:
    command_exception() {}
    virtual ~command_exception() {}

    virtual bool is_equal(const std::string &str) const
	{ return _id == str; }
	
protected:

    std::string _id;    /* identyfikator wyjatku */
};

/*
 * klasa podstawowa dla kazdej komendy
 */
class command_base {
public:

    command_base(const std::string &str, std::vector<std::string> &events);
    virtual ~command_base();

    virtual void run(const std::vector<std::string> &argv)=0;
    virtual std::string help_short(void)=0;
    virtual std::string help_long(void)=0;
    virtual const std::vector<std::string> &completition(void)=0;
    virtual const std::vector<std::string> &complete(const std::string &arg)=0;

    const std::string &name(void)
	{ return _id; }

    bool is_equal(const std::string &str) const;

protected:

    std::string _id;    /* nazwa komendy */
    std::vector<std::string> &_events;
    std::vector<std::string> _completition;
};


bool operator==(const command_base &c, const std::string &str);


#endif /* __COMMAND_HH__ */
