/*
 *      Program do przegladania logow z ekg.
 *      Wyszukiwanie stringa w wiadomosciach.
 *
 *
 *      copyright (c) 2003 by Lukasz Bartnik
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License Version 2 as
 *  published by the Free Software Foundation.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */


#include "find.hh"

#include "ui.hh"

extern "C" {
    #include <sys/timeb.h>
}

/*
 * command_find::run()
 *
 * wywoluje dzialanie dla komendy quit - argumenty nie sa sprawdzane
 */

command_find::lf command_find::_last_find;


bool operator==(const find_mask &fm, std::string &str)
{       return fm.isequ(str);   }

bool operator==(std::string &str, const find_mask &fm)
{       return fm.isequ(str);   }

void operator+=(find_mask &fm, char ch)
{       return fm.addchar(ch);    }


void command_find::run(const std::vector<std::string> &argv) {
    int log=0, chat=0, msg=0;
    if(_db->size() == 0) {
	_mw->print("Baza jest pusta.",
	"\\c010:\\c100:\\c110:\\d ");
	return;
    }

    std::string searched;

    _db->get_uinlog(_last_find.log)->get_chat(_last_find.chat).get_message(_last_find.msg).highlighted = false;

    /* ### Analiza argumentow komendy. ### */
    if(argv.size() == 0) {
	_mw->print("Sprawd help find.", "\\c010:\\c100:\\c110:\\d ");
	return;
    }
    else if(argv.size() == 1) {
	if(!cfg.check_letter_case())
	    for(unsigned int i=0; i<argv[0].size(); ++i)
		searched += tolower(argv[0][i]);
	else
	    searched = argv[0];

	if(_last_find.str != searched) {
	    _last_find.log = 0;
	    _last_find.chat = 0;
	    _last_find.msg = 0;
	    _last_find.offset = 0;
	    _last_find.str = searched;
	}
    }
    else if(argv.size() == 2) {
	if(argv[0] != "-c") {
	    std::string tmp = "Nieznany parametr: " + argv[0];
	    _mw->print(tmp.c_str(), "\\c010:\\c100:\\c110:\\d ");
	    return;
	}
	else {
	    if(!cfg.check_letter_case())
		for(unsigned int i=0; i<argv[1].size(); ++i)
		    searched += tolower(argv[1][i]);
	    else
		searched = argv[1];

	    /* moze zdarzyc sie, ze -c bedzie dotyczylo tej samej rozmowy */
	    if(_last_find.log != _ui->_chosen_chat.log ||
	       _last_find.chat != _ui->_chosen_chat.chat ||
	       _last_find.str != searched)
	    {
		_last_find.log = _ui->_chosen_chat.log;
		_last_find.chat = _ui->_chosen_chat.chat;
		_last_find.msg = 0;
		_last_find.offset = 0;
		_last_find.str = searched;
	    }
	}
    }
    else {
	_mw->print("Dopuszczalne s co najwyej dwa argumenty, z ktrych "
		   "pierwszy to parametr, a drugi to wyszukiwany cig.",
		   "\\c010:\\c100:\\c110:\\d ");
	return;
    }

    timeb tb,te;
    int sec, msec;
    ftime(&tb);

    /* ### Wlasciwy kod wyszukiwania ### */
    _offset = 0;
    for(log = _last_find.log; log!=_db->size(); ++log) {
	uinlog_t *uinlog = _db->get_uinlog(log);

	for(chat = _last_find.chat; chat!=uinlog->size(); ++chat) {
	    chat_t *chptr = &uinlog->get_chat(chat);

	    for(msg = _last_find.msg; msg!=chptr->size(); ++msg) {
		/* usuwamy ze stringa znaki, ktore przeszkadzaja w
		   wyszukiwaniu */
		std::string msg__ = chptr->get_message(msg).str;
		unsigned int off=0;
		while((off = msg__.find("\\n")) != std::string::npos)
		    msg__.erase(off, 2);
		while((off = msg__.find("\\r")) != std::string::npos)
		    msg__.erase(off, 2);
		while((off = msg__.find("\n")) != std::string::npos)
		    msg__.erase(off, 1);

		if(!cfg.check_letter_case()) {
		    for(unsigned int i=0; i<msg__.size(); ++i)
			msg__[i] = tolower(msg__[i]);
		}

		_offset = msg__.find(_last_find.str, _last_find.offset);
		if(_offset != std::string::npos) {
		    find_str(chptr->get_message(msg).str, _last_find.str, _last_find.offset);

		    _last_find.offset = _offset+1;
		    chptr->get_message(msg).highlighted = true;

		    _last_find.log = log;
		    _last_find.chat = chat;
		    _last_find.msg = msg;
		    goto END;
		}
		_last_find.offset = 0;
	    }
	    _last_find.msg = 0;
	}
	_last_find.chat = 0;
    }

END:
    /* obliczamy czas wyszukiwania */
    ftime(&te);
    msec = te.millitm - tb.millitm;
    sec = te.time - tb.time;
    if(msec < 0) {
	--sec;
	msec+=1000;
    }

    if(_offset != std::string::npos) {
	std::string tmp = "Znaleziono cig \\c710[\\d" + _last_find.str;
	tmp += "\\c710]\\d: " + _db->get_uinlog(log)->nick();
	tmp += '/' + string_cast(_db->get_uinlog(log)->uin());
	tmp += ", czas szukania: \\c710" + string_cast(sec);
	tmp += "." + string_cast(add_zero(msec,3)) + "\\d s.";

	_mw->print(tmp.c_str(), "\\c010:\\c400:\\c410:\\d ");
	_mw->show_last();

	throw found_string_exception(log,chat,msg,_offset,_len);
    } else {
	std::string tmp = "Nie znaleziono cigu \\c710[\\d" + _last_find.str;
	tmp += "\\c710]\\d (czas: \\c710";
	tmp += string_cast(sec) + "." + string_cast(add_zero(msec,3)) + "\\d sekund).";

	_mw->print(tmp.c_str(), "\\c010:\\c400:\\c410:\\d ");
	_mw->show_last();

	/* czyscimy zapamietane wspolrzedne, jesli nic nie znalazl */
	_last_find.log = 0;
	_last_find.chat = 0;
	_last_find.msg = 0;
	_last_find.offset = 0;
	_last_find.str = "";

	throw found_string_exception(0,0,0,0,0);
    }
}



void command_find::find_str(const std::string &src, const std::string &m, unsigned int off) {
    unsigned int i=0;
    std::string match;

    if(!cfg.check_letter_case())
	for(unsigned int j=0; j<m.size(); ++j)
	    match += (char)(tolower(m[j]));
    else
	match = m;

    find_mask fm(std::string(match.size(), 'a'));

    for(i=off; i<src.size(); ++i) {
	fm += src[i];
	if(fm.isequ(match)) {
	    _offset = i-fm.size()+1;
	    _len = fm.size();
	    return;
	}
    }

    _offset = std::string::npos;
    _len = 0;
}


const std::vector<std::string>& 
command_find::completition(void) {
    return _completition;
}


const std::vector<std::string> &
command_find::complete(const std::string &arg) {
    if(!arg.size()) {
	_completition.clear();
	_completition.push_back("-c");
	return _completition;
    }
    if(arg == "-") {
	_completition.clear();
	_completition.push_back("c");
	return _completition;
    } else if(arg[0] == '-') {
	_completition.clear();
	return _completition;
    }

    return _completition;
}



/*
 * pomoc
 */
std::string command_find::help_short(void) {
    return " \\c710find\\d    - znajd w rozmowach podany string";
}

std::string command_find::help_long(void) {
    return "\\c710find\\d [\\c710cig\\d] - wyszukuje we wszystkich wiadomociach podany cig; "
	     "jeli [cig] zostanie powtrzony po raz drugi, wyszuka nastpne "
	     "wystpienie\n\n"
	     "   Parametry:\n"
	     " -c [cig] - jeli otwarta jest jaka rozmowa, zaczyna wyszukiwanie "
	     "od tej rozmowy\n\n"
	     "Znaki specjalne poprzedza si znakiem \\; poszukiwany cig mona te "
	     "uj w znaki cudzysowu, np. \"wyszukiwany cig\".\n"
	     "W tej chwili wielko liter nie ma znaczenia: \"Scibi\" == \"scibi\".";
}
