/*
 *      Program do przegladania logow z ekg.
 *      Rozne komendy.
 *
 *
 *      copyright (c) 2003 by Lukasz Bartnik
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License Version 2 as
 *  published by the Free Software Foundation.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#ifndef __COMMAND_LIST_HH__
#define __COMMAND_LIST_HH__


#include "command.hh"
#include "errors.hh"
#include "mainwin.hh"
#include "dbase.hh"

class command_list : public command_base {
public:

    command_list(const std::string &str, std::vector<std::string> &events, mainwin *mw, dbase_t *db)
		:command_base(str, events),_mw(mw),_db(db),_all_chats(0),_all_msgs(0) {}
    virtual ~command_list() {}

    virtual void run(const std::vector<std::string> &argv);
    virtual std::string help_short(void);
    virtual std::string help_long(void);
    virtual const std::vector<std::string> &completition(void);
    virtual const std::vector<std::string> &complete(const std::string &arg);

private:
    struct l_ent {
	l_ent(const std::string &n, int a, int b, int c):chats(b),msgs(c),uin(a),nick(n) {}

	int chats,msgs,uin;
	std::string nick;
    };

    struct sort_nick { bool operator()(const l_ent &a, const l_ent &b) { return a.nick < b.nick; } };
    struct sort_uin  { bool operator()(const l_ent &a, const l_ent &b) { return a.uin < b.uin; } };
    struct sort_chat { bool operator()(const l_ent &a, const l_ent &b) { return a.chats > b.chats; } };
    struct sort_msg  { bool operator()(const l_ent &a, const l_ent &b) { return a.msgs > b.msgs; } };

    void build_list(void);

    mainwin *_mw;
    dbase_t *_db;
    int _all_chats,_all_msgs;
    std::vector<l_ent> _list;
};


#endif /* __COMMAND_LIST_HH__ */
