/*
 *      Program do przegladania logow z ekg.
 *      Wczytywanie konfiguracji z plikow ekglog.
 *
 *
 *      copyright (c) 2003 by Lukasz Bartnik
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License Version 2 as
 *  published by the Free Software Foundation.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */


#include "files.hh"

std::map<std::string,std::string> read_ekg_config(void) {
    char *home = getenv("HOME");
    if(!home)
	error_exit("Nie udao si pobra zmiennej $HOME.");

    std::string cpath = home;
    cpath += "/.gg/config";

    std::ifstream cfile(cpath.c_str());
    if(!cfile)
	error_exit("Nie udao si otworzy pliku %s.", cpath.c_str());


    std::map<std::string,std::string> var;
    while(cfile) {
	std::string id;
	char ch;

	while((ch = cfile.get()) != ' ')
	    id += ch;

	std::string val;
	while((ch = cfile.get()) != '\n')
	    val += ch;

	var[id] = val;

	ch = cfile.get();
	if(cfile)
	    cfile.putback(ch);
    }

    if(var.find("log_path") != var.end()) {
	if(var["log_path"][0] == '~')
	    var["log_path"].replace(0,1,home);
    }
    else
	error_exit("W pliku %s nie ma zmiennej log_path.",cpath.c_str());

    if(var.find("log") == var.end())
	error_exit("W pliku %s nie ma zmiennej log.",cpath.c_str());

    if(var.find("uin") == var.end())
	error_exit("W pliku %s nie ma zmiennej uin.",cpath.c_str());

    return var;
}


std::vector<std::string>
prepare_file_list(std::map<std::string,std::string> var) {
    int log;
    std::istringstream tmp(var["log"]);
    tmp >> log;
    if(!tmp)
	error_exit("Zy format zmiennej log (log == %s).",var["log"].c_str());

    std::vector<std::string> files;
    if (log == 6)
	// dir,gzip - to wci katalog - mona list plikw tak samo wczyta
	log = 2;
    switch(log) {
    case 0:
	error_exit("Zmienna log rwna 0. Nie ma logw rozmw.");
	break;
    case 1:
	files.push_back(var["log_path"]);
	break;
    case 2:
	DIR *dir;
	struct dirent *dent;

	if((dir = opendir(var["log_path"].c_str())) == NULL)
	    error_exit("Nie moge otworzyc katalogu: %s.\n",var["log_path"].c_str());

	while((dent = readdir(dir)) != NULL) {
	    if( !strcmp( dent->d_name, "." ) ||
		!strcmp( dent->d_name, ".." ) ||
		!strcmp( dent->d_name, "sms" ) )
	    continue;

	    std::string name = var["log_path"] + '/';
	    name += dent->d_name;
	    files.push_back(name);
	}

	closedir(dir);
	break;
    case 5:
	// file,gzip - zmienna log_path powinna zawiera nazw pliku bez
	// rozszerzenia, ktre jest automatycznie dodawane przez ekg.
	files.push_back(var["log_path"]+".gz");
	break;
    default:
	error_exit("Nieprawidowa warto zmiennej log (log == %d).",log);
    }

    return files;
}


/*
 * get_msg()
 *
 * pobiera ze strumienia wiadomosc
 * jesli nastapil blad ustawiana jest flaga msg_t.bad
 *
 *  - in - strumien wejsciowy
 */
#define get_entry(c)       i=0;\
			    while((ch = in.get()) != (c) && in  && (i<8192) && (ch != '\n'))\
				buf[i++] = ch;\
			    buf[i] = 0;

msg_t* get_msg(std::istream &in) {
    char ch, buf[8192];
    int i;
    static msg_t msg;

    get_entry(',');

    if(!strcmp(buf,"chatsend"))
	msg.type = CHAT_SEND;
    else if(!strcmp(buf,"chatrecv"))
	msg.type = CHAT_RECV;
    else if( !strcmp(buf,"msgsend"))
	msg.type = MSG_SEND;
    else if(!strcmp(buf,"msgrecv"))
	msg.type = MSG_RECV;
    else if(!strcmp(buf,"status"))
	msg.type = STATUS;
    else {
	while((ch = in.get()) != '\n' && in);

	if(in) {
	    msg_error err;
	    err.str = "Bdny wpis msg.type (";
	    err.str += buf;
	    err.str += ").";
	    throw(err);
	}

	return NULL;
    }

    get_entry(',');
    msg.uin = atoi(buf);

    get_entry(',');
    strcpy(msg.nick, buf);


    if(msg.type == STATUS) {
	/* wczytujemy adres - w tej chwili nieuzywane */
	get_entry(',');
    }

    get_entry(',');
    msg.time = atoi(buf);
    
    if(msg.type == STATUS) {
	/* wczytujemy zmiane statusu */
	get_entry(',');
	if(!strcmp(buf,"avail"))
	    msg.rtime = STAT_AVAIL;
	else if(!strcmp(buf,"busy"))
	    msg.rtime = STAT_BUSY;
	else if( !strcmp(buf,"invisible"))
	    msg.rtime = STAT_INVISIBLE;
	else /* if(!strcmp(buf,"notavail")) */
	    msg.rtime = STAT_NOTAVAIL;

	/* jako, ze get_entry() to makro, to w ch mamy ostatnio wczytany znak */
	if(ch == ',') {
	    /* doczytujemy tresc opisu */
	    get_entry('\n');
	    if(buf[0] == '\"' && i) {
		buf[i-1] = 0;
		strcpy(msg.str, buf+1);
	    }
	    else
		strcpy(msg.str,buf);
	} else
	    return NULL;        /* nie dodajemy stausow bez opisow */

	return &msg;    /* tutaj konczy sie funkcja, bo caly status juz wczytany */
    } else if(msg.type == MSG_RECV || msg.type == CHAT_RECV) {
	get_entry(',');
	msg.rtime = atoi(buf);
    }
    else
	msg.rtime = 0;

    get_entry('\n');
    if(buf[0] == '\"' && i) {
	buf[i-1] = 0;
	strcpy(msg.str, buf+1);
    }
    else
	strcpy(msg.str,buf);

    return &msg;
}


/*
 * read_ekg_userlist()
 *
 * wczytuje do mapy cala liste z .gg/userlist
 */
std::map<int,std::string> read_ekg_userlist(void) {
    std::map<int,std::string> tmp;
    char *home = getenv("HOME");
    if(!home)
	error_exit("Nie mona odczyta zmiennej $HOME.");

    std::string path = home;
    path += "/.gg/userlist";

    std::ifstream in(path.c_str());
    if(!in)
	return tmp;

    while(in) {
	char ch, buf[8192];
	int i, uin, j;
	std::string nick;

	for(j=0; j<3; ++j) {
	    get_entry(';')
	}
	
	if(in) {
	    get_entry(';');
	    nick = buf;

	    for(j=0; j<2; ++j) {
		get_entry(';')
	    }

	    get_entry('\n')
	    if(buf[i-1] == 13)
		buf[i-1] = 0;

	    uin = atoi(buf);

	    tmp[uin] = nick;
	}
    }

    return tmp;
}
