/*
 *      Program do przegladania logow z ekg.
 *      Glowne okno
 *
 *
 *      copyright (c) 2003 by Lukasz Bartnik
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License Version 2 as
 *  published by the Free Software Foundation.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */


#include "mainwin.hh"


mainwin::mainwin(const char *title, int width, int height,
	     int fg, int bold, int bg,
	     int history_lenght)
:window(title,width,height,1,0,1,0,fg,bold,bg),_history(history_lenght),
 _history_overwrite(true),_screen_start(0),_more(false),
 _found(-1),_found_offset(0),_found_lenght(0)
{
}


mainwin::~mainwin() {}


/*
 * mainwin::refresh()
 *
 * odswiez tresc okna
 */
#define delim1 (_screen_start + vsize())
#define delim2 ((int)(_history.size()))
void mainwin::refresh(void) {
    if(!_on_screen || !_win || !_redraw)
	return;

    werase(_win);

    std::string title = _title;
    for(std::vector<std::string>::iterator i=_info.begin();
	i!=_info.end(); ++i )
    {
	title += ' ' + *i;
    }

    if(title.size()) {
	wattron(_win, color_pair(COLOR_WHITE,_act?1:0,COLOR_BLUE));

	wmove(_win,0,0);
	for(int i=0; i<_width; ++i)
	    waddch(_win,' ');

	wmove(_win,0,(_width-title.size())/2);
	wprintw(_win,const_cast<char*>(title.c_str()));

	wattroff(_win, color_pair(COLOR_WHITE,_act?1:0,COLOR_BLUE));
    }

    int i;
    for(i=_screen_start; i<delim1 && i<delim2; ++i) {
	wmove(_win,i+_margin_top-_screen_start,_margin_left);
	wprintl(_win,_history[i].c_str(),color_pair(_fg,_bold,_bg));
    }

    if(i == _history.size())
	_more = false;

    _redraw = false;
    wnoutrefresh(_win);
}


/*
 * mainwin::print()
 *
 * dodaj linijke do historii okna
 */
void mainwin::print(const std::string &str, const std::string &fmt) {
    int fmtlen = line_lenght(fmt);      //format funkcji, nie okna
    unsigned int k=0;
    int len=fmtlen;     // pierwsza linia tez musi byc zainicjalizowana...
    std::string tmp;
    unsigned int print_size = str.size(), i = print_size-1;

    /* pozbywamy sie koncowych spacji */
    while(i>0) {
	if(str[i] == ' ')
	    --print_size;
	else
	    i=1;

	--i;
    }

    while(k < str.size()) {
	switch(str[k]) {
	    case '\\':
		if(++k == str.size()) {
		    tmp += '\\';
		    add_to_history(fmt+tmp);
		    return;
		}
		switch(str[k]) {
		    case 'c':
			tmp += '\\' + str.substr(k, 4);
			k+=4;
			break;
		    case 'd':
			tmp += "\\d";
			++k;
			break;
		    case 'n':
			add_to_history(fmt+tmp);
			tmp.erase();
			len = fmtlen;
			++k;
			break;
		    case 'r':
			++k;
			break;
		    case '\'':
		    case '\\':
		    case '\"':
			tmp += str[k];
			++k;
			++len;
			break;
		    default:
			tmp += '\\';
			tmp += str[k];
			++k;
			++len;
			break;
		}
		break;
	    case '\n':
		add_to_history(fmt+tmp);
		tmp.erase();
		len = fmtlen;
		++k;
		break;
	    default:
		tmp += str[k];
		++len;
		++k;
	}/* switch(*k) */
	if(len == (hsize()) /* -3, bo mamy znaki " | " */
	   || k == print_size)
	{
	    if(k == print_size)
		add_to_history(fmt+tmp);
	    else {
		int j=tmp.size()-1;
		while((tmp[j] != ' ' || tmp[j] == '\n') && j)
		    --j;

		if(!j)
		    add_to_history(fmt+tmp);
		else {
		    int diff = tmp.size() - j;
		    tmp.erase(j,diff);
		    k -= diff-1;
		    add_to_history(fmt+tmp);
		}
	    }

	    tmp.erase();
	    len = fmtlen;
	}
    }/* while() */

    _redraw = true;
    _more = true;
}


/*
 * mainwin::add_to_history()
 *
 * dodaje do historii okna linie wywolujac odpowiednia
 * metode historii w zaleznosci od flagi okna
 *
 *  - str - string do dodania
 */
#include <fstream>
void mainwin::add_to_history(const std::string &str) {
    if(_history_overwrite)
	_history.insert(str);
    else
	_history.add(str);
}


/*
 * mainwin::add_chat()
 *
 * laduje historie okna na nowo wpisujac zadany zapis rozmowy
 * zaklada sie poprawnosc wprowadzanych danych
 *
 *  - chat - obiekt do dodania
 */
#define HIGHLIGHT "\\c007"

#define prepare_highlighted std::string str = chat[i].str;\
			    str.insert(_found_offset, HIGHLIGHT);\
			    for(unsigned int i=_found_offset+1; i<_found_offset+_found_lenght+5;++i) {\
				if(str[i] == ' ' && (i+1)<str.size())\
				    str.insert(i+1, HIGHLIGHT);\
				if((str.substr(i,2) == "\\n") || (str.substr(i,2) == "\\r"))\
				    str.insert(i+2, HIGHLIGHT);\
			    }\
			    str.insert(_found_offset + _found_lenght + 5, "\\d");



void mainwin::add_chat(const Chat &chat) {
    /* poprzedni tytul jest juz niepotrzebny */
    _title.erase();

    _info.clear();
    _info.push_back(chat.nick() + ", ");
    _info.push_back(format_date(chat.start()) + std::string(", "));
//    _info.push_back(format_date(chat.end()) + std::string(", "));
    _info.push_back("wiadomoci: " + string_cast(chat.size()));

    set_history_overwrite(false);
    for(int i=0; i<chat.size(); ++i) {
	if(i == _found) /* podswietlanie dla komendy find */
	    _screen_start = _history.size();

	if(chat[i].type == MSG_RECV || chat[i].type == CHAT_RECV) {
	    print("\\c600.--\\d " + chat.nick() + '/' +
		    string_cast(chat.uin()) + "\\c600 (" +
		    time_string(chat[i].time) + ") --- -- -");

	    if(chat[i].highlighted) {
		    std::string str = chat[i].str;
		    unsigned mv = 0;
		    str.insert(_found_offset, HIGHLIGHT);
		    for(unsigned int i=_found_offset+1; i<_found_offset+_found_lenght+5+mv;++i) {
			if(str[i] == ' ' && (i+1)<str.size()) {
			    str.insert(i+1, HIGHLIGHT);
			    mv += 5;
			}
			if((str.substr(i,2) == "\\n") || (str.substr(i,2) == "\\r")) {
			    str.insert(i+2, HIGHLIGHT);
			    mv += 5;
			}
		    }
		    str.insert(_found_offset + _found_lenght + 5 + mv, "\\d");

		print(str, "\\c600|\\d ");
	    } else
		print(chat[i].str, "\\c600|\\d ");

	    print("\\c600`----- ---- --- -- -");
	}
	else if(chat[i].type == MSG_SEND || chat[i].type == CHAT_SEND) {
	    print("\\c400.--\\d " + cfg.get_nick() + '/' +
		    string_cast(cfg.get_uin()) + "\\c400 (" +
		    time_string(chat[i].time) + ") --- -- -");

	    if(chat[i].highlighted) {
		    std::string str = chat[i].str;
		    unsigned mv = 0;
		    str.insert(_found_offset, HIGHLIGHT);
		    for(unsigned int i=_found_offset+1; i<_found_offset+_found_lenght+5+mv;++i) {
			if(str[i] == ' ' && (i+1)<str.size()) {
			    str.insert(i+1, HIGHLIGHT);
			    mv += 5;
			}
			if((str.substr(i,2) == "\\n") || (str.substr(i,2) == "\\r")) {
			    str.insert(i+2, HIGHLIGHT);
			    mv += 5;
			}
		    }
		    str.insert(_found_offset + _found_lenght + 5 + mv, "\\d");

		print(str, "\\c400|\\d ");
	    } else
		print(chat[i].str, "\\c400|\\d ");

	    print("\\c400`----- ---- --- -- -");
	}
    }
    set_history_overwrite(true);
    _found = -1;

    /* jesli bylo _found != -1, to trzeba sprawdzic, gdzie zaczyna sie ekran */
    while(_history.size() - _screen_start < vsize() && _screen_start > 0)
	--_screen_start;
}


/*
 * mainwin::add_stats()
 *
 * dodaje do okna liste statusow
 *
 *  - s - obiekt statusow
 */
void mainwin::add_stats(const Stats& s) {
    /* poprzedni tytul jest juz niepotrzebny */
    _title.erase();

    _info.clear();
    _info.push_back(s.nick() + ", ");
    _info.push_back("statusw: " + string_cast(s.size()));

    set_history_overwrite(false);
    for(std::vector<message>::const_iterator i=s.stats().begin();
	i!=s.stats().end(); ++i)
    {
	std::string str = format_date(i->time);
	str += " \\c010:\\c600:\\c610:\\c710 \"\\d";
	str += i->str.c_str();
	str += "\\c710\"\\d";
	print(str.c_str());
    }
    set_history_overwrite(true);
}


/*
 * mainwin::clear()
 *
 * czysci cale okno, historie, zeruje indeksy
 */
void mainwin::clear(void) {
    _history.clear();
    _screen_start = 0;
}


/*
 * mainwin::showlast()
 *
 * ustawia wyswietlanie tak, by widac bylo najmlodsza linie
 */
void mainwin::show_last(void) {
    if((int)_history.size() < vsize())
	return;

    _screen_start = ((int)_history.size())-vsize();
    _redraw = true;
}


/*
 * mainwin::actup(), actdown(), actpgup(), actpgdown()
 *
 * cztery metody przesuwajace podswietlenie w liscie
 */
void mainwin::up(void) {
    if(_screen_start == 0 || (int)_history.size() < vsize())
	return;

    --_screen_start;
    _redraw = true;
}

void mainwin::down(void) {
    if((int)_history.size() < vsize())
	return;

    if(_screen_start == ((int)_history.size())-vsize())
	return;

    ++_screen_start;
    _redraw = true;
}

void mainwin::pgup(void) {
    for(int i=0; i<vsize(); ++i)
	up();
}

void mainwin::pgdown(void) {
    for(int i=0; i<vsize(); ++i)
	down();
}
