/*
 *      Program do przegladania logow z ekg.
 *      Interfejs uzytkownika.
 *
 *
 *      copyright (c) 2003 by Lukasz Bartnik
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License Version 2 as
 *  published by the Free Software Foundation.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */


#include "ui.hh"


/*
 * ui::ui()
 *
 * inicjalizuje wszelkie ustawienia ncurses, tworzy okna
 */
#define WIDTH cfg.list_width()
ui_t::ui_t(dbase_t &db):_db(db) {

    initscr();
    cbreak();
    noecho();

    start_color();
    /* kolorki - pozyczone z ekg -> dev.null.pl/ekg/ */
    init_pair(7, COLOR_BLACK, COLOR_BLACK);  /* mae obejcie domylnego koloru */
    init_pair(1, COLOR_RED, COLOR_BLACK);
    init_pair(2, COLOR_GREEN, COLOR_BLACK);
    init_pair(3, COLOR_YELLOW, COLOR_BLACK);
    init_pair(4, COLOR_BLUE, COLOR_BLACK);
    init_pair(5, COLOR_MAGENTA, COLOR_BLACK);
    init_pair(6, COLOR_CYAN, COLOR_BLACK);
						    
#define __init_bg(x, y) \
    init_pair(x, COLOR_BLACK, y); \
    init_pair(x + 1, COLOR_RED, y); \
    init_pair(x + 2, COLOR_GREEN, y); \
    init_pair(x + 3, COLOR_YELLOW, y); \
    init_pair(x + 4, COLOR_BLUE, y); \
    init_pair(x + 5, COLOR_MAGENTA, y); \
    init_pair(x + 6, COLOR_CYAN, y); \
    init_pair(x + 7, COLOR_WHITE, y);

    __init_bg(8, COLOR_RED);
    __init_bg(16, COLOR_GREEN);
    __init_bg(24, COLOR_YELLOW);
    __init_bg(32, COLOR_BLUE);
    __init_bg(40, COLOR_MAGENTA);
    __init_bg(48, COLOR_CYAN);
    __init_bg(56, COLOR_WHITE);
#undef __init_bg


    _kontakty = new menulist("Kontakty", WIDTH, (LINES-2)/2, COLS-WIDTH, 0, COLOR_GREEN, 1, 0);
    _rozmowy = new menulist("Rozmowy", WIDTH, (LINES-2)/2, COLS-WIDTH, (LINES-2)/2, COLOR_GREEN, 1, 0);

    _win = newwin(LINES-2, COLS-WIDTH, 0, 0);

    _mainwin = new mainwin("Czat",COLS-WIDTH, LINES-2, 7,0,0, 1000);
    _mainwin->set_ncurses_window(_win);

    _info = new mainwin("Info",COLS-WIDTH, LINES-2, 7,0,0, 1000);
    _info->set_ncurses_window(_win);

    _status = new status(0, LINES-2, COLS);

    for(int i=0; i<db.size(); ++i) {
	_kontakty->append(db.get_uinlog(i)->nick(), i);
    }

    _input = new input(0, LINES-1, COLS);
    register_command(new command_quit("quit", _events));
    register_command(new command_list("list", _events, _info, &db));
    register_command(new command_find("find", _events, _info, &db, this));
    register_command(new command_help("help", _events, &_commands, _info));

    key[0] = 0;
    key[1] = 0;
    
}


/*
 * ui_t::~ui_t()
 *
 * konczy dzialanie interfejsu, niszczy utworzone okna
 */
ui_t::~ui_t(void) {
    delete _rozmowy;
    delete _kontakty;
    delete _status;
    delete _mainwin;
    delete _info;
    delete _input;

    for(std::vector<command_base*>::iterator i=_commands.begin();
	i!=_commands.end(); ++i)
    {
	delete *i;
    }

    delwin(_win);
    endwin();
}


/*
 * ui_t::main_loop()
 *
 * glowna petla programu
 *
 *  - info - tekst informacyjny z zewnatrz petli
 */
#define KONTAKTY 0
#define ROZMOWY 1
#define MWIN 2
#define INPUT 3
#define INFO 4

#define set_chat_list(no) {\
			    uinlog = _db.get_uinlog(no);\
			    if(uinlog->stats.size())\
				_rozmowy->append("Statusy",-1);\
			    for(int i=uinlog->size()-1; i>-1; --i) {\
				_rozmowy->append(format_date((*uinlog)[i].start()), i);\
			    }\
			  }


void ui_t::main_loop(const std::string &info) {
    int ch;
    window_base *act = _input;
    bool slow = false, ekglog_quit = false;
    uinlog_t *uinlog;

    _mainwin->set_display(false);
    _info->set_display(true);
    _info->passive();
    
    set_chat_list(_kontakty->get_actid());

    _info->print("\\c710ekglog\\d, wersja \\c710" WERSJA
		 "\\d. copyrights (c) 2003 by \\c710ukasz \\c710Bartnik\\d.\n"
		 "\\c110Wcinij \\c110F1 \\c110aby \\c110uzyska \\c110pomoc.\\d",
		 "\\c010:\\c200:\\c210:\\d ");
    _info->print(("\n"+info).c_str(), "\\c010:\\c200:\\c210:\\d ");

    _status->set_title("\\c714Linia komend\\d");

    refresh();

    while(!ekglog_quit) {
	ch = _input->getkb();

	/* utrzymujemy historie kolejnych klawiszy */
	key[0] = key[1];
	key[1] = ch;

	switch(ch) {
	    case KEY_UP:        _events.push_back("keyboard_up");       break;
	    case KEY_DOWN:      _events.push_back("keyboard_down");     break;
	    case KEY_PPAGE:     _events.push_back("keyboard_pgup");     break;
	    case KEY_NPAGE:     _events.push_back("keyboard_pgdown");   break;
	    case 0x09:          /* TAB -> dopelnianie */
		    if(act == _input)
			try_complete();

		    break;
	    case 0x1B:     /* ALT + cos */
		    key[0] = key[1];
		    switch(key[1] = getch()) {
			case '1':
			    _events.push_back("make_all_passive");
			    _events.push_back("switchto_1");
			    break;
			case '2':
			    _events.push_back("make_all_passive");
			    _events.push_back("switchto_2");
			    break;
			case '3':
			    _events.push_back("make_all_passive");
			    _events.push_back("switchto_3");
			    break;
			case '4':
			    _events.push_back("make_all_passive");
			    _events.push_back("switchto_4");
			    break;
			case '5':
			    _events.push_back("make_all_passive");
			    _events.push_back("switchto_5");
			    break;
		    }
		    break;
	    case 0x0A:  /* ENTER */
		    if(act == _kontakty)
			_events.push_back("keyboard_enter_switchto_kontakty");
		    else if(act == _rozmowy)
			_events.push_back("keyboard_enter_switchto_rozmowy");
		    else if(act == _input)
			_events.push_back("keyboard_add_to_input");

		    break;

//            case 'f':
	    case KEY_F(1):
		_events.push_back("keyboard_f1");
		break;

	    case KEY_F(2):
		_events.push_back("keyboard_f2");

		    break;

	    case '`':
	    case KEY_F(3):
		_events.push_back("test");

		    break;

	    case 4:             /* Ctrl+D */
	    case KEY_F(10):
		    ekglog_quit = true;

		    break;

	    default:
		if(act == _input)
		    _events.push_back("keyboard_add_to_input");
		break;
	}

	/* ### OBSLUGA ZDARZEN Z WEKTORA ZDARZEN */

	for(unsigned int i=0; i<_events.size(); ++i)
	{
	    if(_events[i] == "keyboard_add_to_input") {
		_input->add_key(ch);

		try {
		    if(_input->is_new()) { 
			/* jesli pojawila sie nowa komenda */
			bool stop = false;
			using std::string;
			std::vector<string> argv = _input->get_argv();

			string command_id = argv[0];
			argv.erase(argv.begin());

			std::vector<command_base*>::iterator i;
			for(i=_commands.begin(); i!=_commands.end() && !stop; ++i)
			    if((**i) == command_id) {
				(*i)->run(argv);
				stop = true;
			    }

			if(!stop) {
			    std::string tmp = "Nieznane polecenie: " + command_id;
			    _info->print(tmp.c_str() , "\\c010:\\c100:\\c110:\\d ");

			    _mainwin->set_display(false);
			    _info->set_display(true);
			    _info->passive();
			    _info->show_last();
			    argv.clear();
			}
		    }
		}/* try */
		catch(found_string_exception &e) {
		    if(e.len == 0) {
			/* nie znaleziono ciagu */
			_mainwin->set_display(false);
			_info->set_display(true);
		    } else {
			_info->set_display(false);

			_mainwin->set_display(true);
			_mainwin->clear();
			_mainwin->set_found(e.msg, e.off, e.len);
			_mainwin->add_chat(_db.get_uinlog(e.log)->get_Chat(e.chat));
			_db.get_uinlog(e.log)->get_chat(e.chat).get_message(e.msg).highlighted = false;

			_kontakty->set_act(e.log);

			_rozmowy->clear();
			set_chat_list(_kontakty->get_actid());
			_rozmowy->set_act(_rozmowy->size()-1-e.chat);

			_chosen_chat.log = _kontakty->get_actid();
			_chosen_chat.chat = _rozmowy->get_actid();
		    }
		}
		catch(quit_exception) {
		    ekglog_quit = true;
		}
	    }
	    else if(_events[i] == "keyboard_up") {
		    act->up();
		    if(act == _kontakty && !slow) {
			_rozmowy->clear();
			set_chat_list(_kontakty->get_actid());
		    }
	    }
	    else if(_events[i] == "keyboard_down") {
		    act->down();
		    if(act == _kontakty && !slow) {
			_rozmowy->clear();
			set_chat_list(_kontakty->get_actid());
		    }
	    }
	    else if(_events[i] == "keyboard_pgup") {
		    act->pgup();
		    if(act == _kontakty && !slow) {
			_rozmowy->clear();
			set_chat_list(_kontakty->get_actid());
		    }
	    }
	    else if(_events[i] == "keyboard_pgdown") {
		    act->pgdown();
		    if(act == _kontakty && !slow) {
			_rozmowy->clear();
			set_chat_list(_kontakty->get_actid());
		    }
	    }
	    else if(_events[i] == "switchto_1") {
		    act = _mainwin;
		    _mainwin->set_display(true);
		    _info->set_display(false);
		    _status->set_title("Okno rozmowy");
		    act->active();
	    }
	    else if(_events[i] == "switchto_2") {
		    act = _info;
		    _mainwin->set_display(false);
		    _info->set_display(true);
		    _status->set_title("Info");
		    act->active();
	    }
	    else if(_events[i] == "switchto_3") {
		    act = _kontakty;
		    _status->set_title("Lista kontaktw");
		    act->active();
	    }
	    else if(_events[i] == "switchto_4") {
		    act = _rozmowy;
		    _status->set_title("Lista rozmw");
		    act->active();
	    }
	    else if(_events[i] == "switchto_5") {
		    act = _input;
		    _status->set_title("\\c714Linia komend\\d");
		    act->active();
	    }
	    else if(_events[i] == "make_all_passive") {
		    _kontakty->passive();
		    _rozmowy->passive();
		    _mainwin->passive();
		    _info->passive();
	    }
	    else if(_events[i] == "keyboard_enter_switchto_kontakty") {
		    _rozmowy->clear();
		    set_chat_list(_kontakty->get_actid());

		    _kontakty->passive();
		    _rozmowy->active();
		    _status->set_title("Lista rozmw");

		    act = _rozmowy;
	    }
	    else if(_events[i] == "keyboard_enter_switchto_rozmowy") {
		    #define current_uinlog _db.get_uinlog(_kontakty->get_actid())
		    _mainwin->clear();
		    if(_rozmowy->get_actid() == -1) {
			/* dodajemy statusy */
			_mainwin->add_stats(Stats(current_uinlog));
		    } else {
			_mainwin->add_chat(current_uinlog->get_Chat(_rozmowy->get_actid()));
		    }
		    #undef current_uinlog
		    _info->set_display(false);
		    _mainwin->set_display(true);
		    _chosen_chat.log = _kontakty->get_actid();
		    _chosen_chat.chat = _rozmowy->get_actid();
	    }
	    else if(_events[i] == "keyboard_f1") {
		    help_keys(_info);

		    _mainwin->set_display(false);
		    _info->set_display(true);
		    _info->active();
		    _info->show_last();
		    _status->set_title("Info");
		    act = _info;
	    }
	    else if(_events[i] == "keyboard_f2") {
		    _input->_line = "help";
		    _events.push_back("keyboard_add_to_input");
		    ch = 10;                
	    }
	    else if(_events[i] == "test") {
		    _input->_line = "find Scibi";
		    _events.push_back("keyboard_add_to_input");
		    ch = 10;                
	    }
	}
	_events.clear();
	refresh();
    }
}


/*
 * ui_t::refresh()
 *
 * odswieza wszystkie okna
 */
void ui_t::refresh(void) {
    check_errors();

    _rozmowy->refresh();
    _kontakty->refresh();
    _mainwin->refresh();
    _info->refresh();

    if(_mainwin->is_displayed()) {
	if(_info->more())
	    _status->set_actstr("Info");
	else
	    _status->set_actstr("");
	_status->set_more(_mainwin->more());
    }
    else {
	if(_mainwin->more())
	    _status->set_actstr("Chat");
	else
	    _status->set_actstr("");
	_status->set_more(_info->more());
    }

    _status->refresh();
    _input->refresh();

    doupdate();
}


/*
 * ui_t::try_comlete()
 *
 * sprawdza, czy mozna dopelnic linie komend
 */
void ui_t::try_complete(void) {
    std::string &tmp = _input->_line,
		command_part;
    unsigned int i=0;

    /* nie dopelniamy w srodku linii oraz w srodku komentarza */
    if((_input->_line_pos != _input->_line.size()) ||
		!line_is_good() )
    {
	return;
    }

    /* jesli to jest pierwszy tab, wyczysc poprzednie dopelnienia */
    if(key[0] != 0x09)
	_cmplt.clear();


    if(key[0] == 0x09 && key[1] == 0x09) {
	if(_cmplt.size() > 1) {
	    std::string cmplt;

	    /* znajdujemy rozmiar kolumny */
	    unsigned int rowsize = 0;
	    for(i=0; i<_cmplt.size(); ++i)
		if(_cmplt[i].size() > rowsize)
		    rowsize = _cmplt[i].size();

	    rowsize += 2;
	    if((int)rowsize < _info->hsize()) {
		/* jesli ekran ma wystarczajaca ilosc miejsca... */
		unsigned int rows = _info->hsize()/rowsize, row=0;

		for(i=0; i<_cmplt.size(); ++i) {
		    cmplt += _cmplt[i];
		    if(!((row+1) % rows))
			cmplt += "\n";
		    else {
			for(int k=rowsize-_cmplt[i].size(); k>0; --k)
			    cmplt += ' ';
		    }
		    ++row;
		}
	    } else {
		for(i=0; i<_cmplt.size(); ++i)
		    cmplt += _cmplt[i] + ' ';
	    }

	    _info->print(cmplt.c_str(), "");
	    _info->set_display(true);
	    _mainwin->set_display(false);
	    _info->show_last();
	    return;
	}
    }

    unsigned int f;
    if((f = tmp.find(" ")) != std::string::npos) {
	_cmplt.clear();

	if(tmp.find(" ", f+1) != std::string::npos)
	    return;     /* dopelniaj tylko pierwszy argument */

	/* uzupelniamy argument - nie komende */
	std::string command = tmp.substr(0,tmp.find_first_of(" "));
	for(std::vector<command_base*>::iterator i=_commands.begin();
		i!=_commands.end(); ++i)
	    if((*i)->is_equal(command)) {
		std::string searched = tmp.substr(tmp.find_last_of(" ")+1,std::string::npos);
		/* probujemy dopelnic */
		_cmplt = (*i)->complete(searched);

		if(_cmplt.size() == 1) {
		    _input->_line += _cmplt[0].substr(searched.size(),
						    std::string::npos) + ' ';
		    _input->_line_pos = _input->_line.size();
		    key[1] = 0;         /* w ten sposob dajemy znac, ze nastapilo dopelnienie */
		    _cmplt.clear();
		} else if(_cmplt.size() > 1)
		    check_partial_completition();

		
		return;
	    }
    } else {
	/* dopelniamy komende */
	_cmplt.clear();
	for(std::vector<command_base*>::iterator i=_commands.begin();
		i!=_commands.end(); ++i)
	    if((*i)->name().find(tmp) == 0)
		_cmplt.push_back((*i)->name());

	if(_cmplt.size() == 1) {
	    _input->_line = _cmplt[0] + ' ';
	    _input->_line_pos = _input->_line.size();
	    key[1] = 0;         /* w ten sposob dajemy znac, ze nastapilo dopelnienie */
	    _cmplt.clear();
	} else if(_cmplt.size() > 1)
	    check_partial_completition();
    }
}


/*
 * ui_t::register_command()
 *
 * rejestruje nowa komende
 * przy zabijaniu okna na kazdym wskazniku wykonywane jest delete
 *
 *  - c - wskaznik na obiekt komendy
 */
void ui_t::register_command(command_base *c) {
    _commands.push_back(c);
}


/*
 * sprawdza poprawnosc linii polecen
 */
bool ui_t::line_is_good(void) {
    std::string &str = _input->_line;
    bool spc = false;
    unsigned int i=0;

    while(i<str.size()) {
	if(str[i] == '\\')
	    ++i;
	else if(str[i] == '\"')
	    spc = !spc;

	++i;
    }
    if(spc)
	return false;

    return true;
}


/*
 * sprawdza, czy nie mozna dopelnic czesci wyrazu
 */
void ui_t::check_partial_completition(void) {
    std::string &tmp = _input->_line;
    std::string searched = tmp.substr(tmp.find_last_of(" ")+1,std::string::npos);
    unsigned int i = searched.size(), size = ~0;
    /* sprawdzamy czy nie mozna dopelnic czesci nazwy */
    std::string complete;
    for(std::vector<std::string>::iterator k=_cmplt.begin();
	    k!=_cmplt.end(); ++k)
	{
	    if(k->size() < size)
		size = k->size();
//            _info->print((*k).c_str(),"");
	}

    bool stop = false;
    for( ;i<size && !stop; ++i) {
	char ch = _cmplt[0][i];

	for(std::vector<std::string>::iterator k=_cmplt.begin();
		k!=_cmplt.end() && !stop; ++k)
	    if((*k)[i] != ch)
		stop = true;

	if(!stop)
	    complete += ch;
    }
    if(complete.size()) {
	_input->_line += complete;
	_input->_line_pos = _input->_line.size();
    }
}


/*
 * ui_t::check_errors()
 *
 * sprawdza wektor z komunikatami bledow i wpisuje wszystkie
 * do okna info
 */
void ui_t::check_errors(void) {
    if(errors.size()) {
	for(std::vector<std::string>::iterator i=errors.begin();
	    i!=errors.end(); ++i)
	{
	    _info->print(*i, "\\c010:\\c100:\\c110:\\d ");
	}
	errors.clear();
    }
}
