/*
 *      Program do przegladania logow z ekg.
 *      Klasa podstawowa okna.
 *
 *
 *      copyright (c) 2003 by Lukasz Bartnik
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License Version 2 as
 *  published by the Free Software Foundation.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#ifndef __WINDOW_HH__
#define __WINDOW_HH__

#include <string>
#include "history.hh"
#include "stuff.hh"


extern "C" {
#include <ncurses.h>
}


class window_base {
public:
    window_base() {}
    virtual ~window_base() {}

    virtual void refresh(void)=0;         /* odswiez okno */
    virtual void up(void)=0;
    virtual void down(void)=0;
    virtual void pgup(void)=0;
    virtual void pgdown(void)=0;

    virtual void active(void)=0;
    virtual void passive(void)=0;
};


class window : public window_base {
public:
    window(const char *title, 
	    int width, int height,
	    int margin_top, int margin_bottom, int margin_left, int margin_right,
	    int fg, int bold, int bg);
    virtual ~window() {}

    void active(void) { _act = true; _redraw = true; }  /* ustaw pasek tytulu na aktywny */
    void passive(void) { _act = false; _redraw = true; }/* ustaw pasek tytulu na pasywny */

    int vsize(void) { return _height - _margin_top - _margin_bottom; }
    int hsize(void) { return _width - _margin_left - _margin_right; }

protected:

    bool redraw(void) { return _redraw; }
    void set_redraw(void) { _redraw = true; }



    WINDOW *_win;               /* okno ncurses */
    int _width, _height;        /* wymiary okna */
    int _margin_top, _margin_bottom, _margin_left, _margin_right;
				/* marginesy dla teksty w oknie */
    int _fg, _bold, _bg;        /* domyslne kolory okna */

    std::string _title;         /* tresc paska tytulu */

    bool _redraw;               /* czy okno nalezy przerysowac */
    bool _act;                  /* czy pasek okna jest aktywny */
};

#endif /* __WINDOW_HH__ */
